;; thinmaker/theme.jl
;; $Id: theme.jl,v 1.17 1999/11/25 23:38:41 john Exp $

;; Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
;; Modifications by Geoffrey Simms <geoff@hrl.com>

(require 'gradient)

;; Commentary:

;; Some diciness in the glyph sizing code, but it seems to work.
;; Also, some features don't seem to take effect until a restart, like
;; titlebar resize and turning transient title labels on and off.

(defgroup thinmaker-colors "Thinmaker Colors")
(defgroup thinmaker-frame "Thinmaker Frame")

(defcustom thinmaker:gradient-type 'vertical
  "Direction of gradient in `thinmaker' frame style."
  :type symbol
	:options (horizontal vertical diagonal)
  :group thinmaker-colors
  :after-set after-setting-frame-option)

(defcustom thinmaker:normal-from-color (get-color "gray90")
  "`From' color of inactive frames in `thinmaker' frame style."
  :type color
	:group thinmaker-colors
  :after-set after-setting-frame-option)

(defcustom thinmaker:normal-to-color (get-color "gray60")
  "`To' color of inactive frames in `thinmaker' frame style."
  :type color
  :group thinmaker-colors
  :after-set after-setting-frame-option)

(defcustom thinmaker:normal-font-color (get-color "black")
  "Font color of inactive frames in `thinmaker' frame style."
  :type color
  :group thinmaker-colors
  :after-set after-setting-frame-option)

(defcustom thinmaker:active-from-color (get-color "steelblue4")
  "`From' color of active frames in `thinmaker' frame style."
  :type color
  :group thinmaker-colors
  :after-set after-setting-frame-option)

(defcustom thinmaker:active-to-color (get-color "steelblue3")
  "`To' color of active frames in `thinmaker' frame style."
  :type color
  :group thinmaker-colors
  :after-set after-setting-frame-option)

(defcustom thinmaker:active-font-color (get-color "yellow2")
  "Font color of active frames in `thinmaker' frame style."
  :type color
  :group thinmaker-colors
  :after-set after-setting-frame-option)

(defcustom thinmaker:save-memory t
  "Use less memory when creating gradients, possibly affecting quality."
  :type boolean
  :group thinmaker-colors
  :after-set after-setting-frame-option)


(defcustom thinmaker:frame-thickness '17
	"Thickness of titlebar frame in the `thinmaker' frame style."
	:type number
	:group thinmaker-frame
	:after-set after-setting-frame-option
	:range (12 . 30))

(defcustom thinmaker:resize-thickness '6
	"Thickness of the bottom resize bar in the `thinmaker' frame style."
	:type number
	:group thinmaker-frame
	:after-set after-setting-frame-option
	:range (4 . 20))

(defcustom thinmaker:transient-thickness '6
	"Thickness of the top bar in the transient frame style."
	:type number
	:group thinmaker-frame
	:after-set after-setting-frame-option
	:range (4 . 20))

(defcustom thinmaker:transient-title nil
	"Toggle text in transient titlebars."
	:type boolean
	:group thinmaker-frame
	:after-set after-setting-frame-option)

(defcustom thinmaker:glyph-size '13
	"Size of glyphs used in buttons in `thinmaker' frame style."
	:type number
	:group thinmaker-frame
	:after-set after-setting-frame-option
	:range (10 . 99))

(defcustom thinmaker:minimize-button-glyph 'box1
	"Which glyph from Zapf Dingbats to use in minimize button."
	:type symbol
	:options (box1 box2 circle1 circle2 triup tridown rhombus rhombi flower sun snow1 snow2 star1 star2  star3 star4 check1 check2 x1 x2 x3 x4 plus1 plus2 reel cross aster1 aster2 aster3)
	:group thinmaker-frame
	:after-set after-setting-frame-option)

(defcustom thinmaker:close-button-glyph 'x1
	"Which glyph from Zapf Dingbats to use in close button."
	:type symbol
	:options (box1 box2 circle1 circle2 triup tridown rhombus rhombi flower sun snow1 snow2 star1 star2  star3 star4 check1 check2 x1 x2 x3 x4 plus1 plus2 reel cross aster1 aster2 aster3)
	:group thinmaker-frame
	:after-set after-setting-frame-option)

(let*
    (
		 
		 (render-bg 
			(lambda 
				(img state)
				(apply 
				 (cond 
					(
					 (eq thinmaker:gradient-type 'diagonal)
					 draw-diagonal-gradient
					 )
					(
					 (eq thinmaker:gradient-type 'horizontal)
					 draw-horizontal-gradient
					 )
					(
					 (eq thinmaker:gradient-type 'vertical)
					 draw-vertical-gradient
					 )
					)
				 img 
				 (cond
					(
					 (eq state nil)
					 (
						list
						thinmaker:normal-from-color
						thinmaker:normal-to-color
						)
					 )
					(
					 (eq state 'focused)
					 (
						list
						thinmaker:active-from-color
						thinmaker:active-to-color
						)
					 )
					(
					 ;; Allow for separate colors for hilited bars if that turns
					 ;; you on.
					 (eq state 'highlighted)
					 (
						list
						thinmaker:active-from-color
						thinmaker:active-to-color
						)
					 )
					(t
					 (
						list
						thinmaker:active-from-color
						thinmaker:active-to-color
						)
					 )
					)
				 )
				(when 
						(> (cdr (image-dimensions img)) 2 )
					(bevel-image img 1 (not (eq state 'clicked)))
					)
				(set-image-border img 1 1 1 1)
				)
			)

     (
			scale 
			(lambda () 
				(if 
						thinmaker:save-memory 2 1
						)
				)
			)
		 
		 

		 (fontcolor
			;; also allows separate highlighted color
			(lambda ()
				(
				 list 
				 thinmaker:normal-font-color
				 thinmaker:active-font-color
				 thinmaker:active-font-color
				 thinmaker:active-font-color 
				 )
				)
			)

		 (framethickness
			(lambda ()
				thinmaker:frame-thickness
				)
			)
		 
		 (resizethickness
			(lambda ()
				thinmaker:resize-thickness
				)
			)

		 (transientthickness
			(lambda ()
				thinmaker:transient-thickness
				)
			)
		 
		 ;; This is a yucky hack. It converts a number from an integer to
		 ;; the string representation of that integer. It only works for
		 ;; numbers between 10 and 99, which is OK for font sizes.
		 (my-int-to-string
			(lambda (num)
				(concat (+ 48 (/ num 10)) (+ 48 (- num (* 10 (/ num 10)))))
				)
			)
		 
		 (glyph-font
			(lambda ()
				get-font (concat "-*-zapf dingbats-*-*-*-*-" (my-int-to-string thinmaker:glyph-size) "-*-*-*-*-*-*-*")
				)
			)

		 ;; This is not the whole font set of Zapf Dingbats, but a highly
		 ;; usable subset, in my opinion.
		 (which-glyph
			(lambda (glyph-name)
				(cond
				 (
					(eq glyph-name 'box1)
					"r"
					)
				 (
					(eq glyph-name 'box2)
					"n"
					)
				 (
					(eq glyph-name 'circle1)
					"m"
					)
				 (
					(eq glyph-name 'circle2)
					"n"
					)
				 (
					(eq glyph-name 'triup)
					"s"
					)
				 (
					(eq glyph-name 'tridown)
					"t"
					)
				 (
					(eq glyph-name 'rhombus)
					"u"
					)
				 (
					(eq glyph-name 'rhombi)
					"v"
					)
				 (
					(eq glyph-name 'flower)
					"a"
					)
				 (
					(eq glyph-name 'sun)
					"b"
					)
				 (
					(eq glyph-name 'snow1)
					"d"
					)
				 (
					(eq glyph-name 'snow2)
					"e"
					)
				 (
					(eq glyph-name 'star1)
					"P"
					)
				 (
					(eq glyph-name 'star2)
					"H"
					)
				 (
					(eq glyph-name 'star3)
					"J"
					)
				 (
					(eq glyph-name 'star4)
					"K"
					)
				 (
					(eq glyph-name 'check1)
					"3"
					)
				 (
					(eq glyph-name 'check2)
					"4"
					)
				 (
					(eq glyph-name 'x1)
					"5"
					)
				 (
					(eq glyph-name 'x2)
					"6"
					)
				 (
					(eq glyph-name 'x3)
					"7"
					)
				 (
					(eq glyph-name 'x4)
					"8"
					)
				 (
					(eq glyph-name 'plus1)
					"9"
					)
				 (
					(eq glyph-name 'plus2)
					";"
					)
				 (
					(eq glyph-name 'reel)
					"'"
					)
				 (
					(eq glyph-name 'cross)
					"="
					)
				 (
					(eq glyph-name 'aster1)
					"i"
					)
				 (
					(eq glyph-name 'aster2)
					"j"
					)
				 (
					(eq glyph-name 'aster3)
					"k"
					)
				 (t
					"5"
					)
				 )
				)
			)
			
		 
		 (iconify-glyph
			(lambda ()
				(which-glyph thinmaker:minimize-button-glyph)
				)
			)
		 
		 (close-glyph
			(lambda ()
				(which-glyph thinmaker:close-button-glyph)
				)
			)
		 
     (frame 
			`(
				;; Titlebar region
				(
				 (renderer . ,render-bg)
				 (render-scale . ,scale)
				 (foreground . ,fontcolor)
				 (text . ,window-name)
				 (x-justify . 10)
				 (y-justify . center)
				 (left-edge . ,(- (framethickness) 1))
				 (right-edge . ,(- (framethickness) 1))
				 (top-edge . ,(- 0 (framethickness) ))
				 (height . ,framethickness)
				 (class . title)
				 )
				;; Left edge outline
				(
				 (background . "gray40")
				 (left-edge . -1)
				 (width . 1)
				 (top-edge . 0)
				 (bottom-edge . 0)
				 )
				;; Right edge outline
				(
				 (background . "black")
				 (right-edge . -1)
				 (width . 1)
				 (top-edge . 0)
				 (bottom-edge . 0)
				 )
				;; Bottom resize bar region
				(
				 (renderer . ,render-bg)
				 (render-scale . ,scale)
				 (left-edge . ,(- (framethickness) 1))
				 (right-edge . ,(- (framethickness) 1))
				 (bottom-edge . ,(- 0 (resizethickness)))
				 (height . ,resizethickness)
				 (class . bottom-border)
				 )
				;; Bottom left corner
				(
				 (renderer . ,render-bg)
				 (render-scale . ,scale)
				 (left-edge . -1)
				 (width . ,framethickness)
				 (bottom-edge . ,(- 0 (resizethickness)))
				 (height . ,resizethickness)
				 (class . bottom-left-corner)
				 )
				;; Bottom right corner
				(
				 (renderer . ,render-bg)
				 (render-scale . ,scale)
				 (right-edge . -1)
				 (width . ,framethickness)
				 (bottom-edge . ,(- 0 (resizethickness)))
				 (height . ,resizethickness)
				 (class . bottom-right-corner)
				 )
				;; Iconify button
				(
				 (renderer . ,render-bg)
				 (render-scale . ,scale)
				 (foreground . ,fontcolor)
				 (font . ,glyph-font)
				 (text . ,iconify-glyph)
				 (x-justify . center)
				 (y-justify . center)
				 (left-edge . -1)
				 (top-edge . ,(- 0 (framethickness)))
				 (width . ,framethickness)
				 (height . ,framethickness)
				 (class . iconify-button)
				 )
				;; Close button
				(
				 (renderer . ,render-bg)
				 (render-scale . ,scale)
				 (foreground . ,fontcolor)
				 (font . ,glyph-font)
				 (text . ,close-glyph)
				 (x-justify . center)
				 (y-justify . center)
				 (right-edge . -1)
				 (top-edge . ,(- 0 (framethickness)))
				 (width . ,framethickness)
				 (height . ,framethickness)
				 (class . close-button)
				 )
				)
			)
		 
     (shaped-frame 
			`(
				;; Titlebar region
				(
				 (renderer . ,render-bg)
				 (render-scale . ,scale)
				 (foreground . ,fontcolor)
				 (text . ,window-name)
				 (x-justify . 10)
				 (y-justify . center)
				 (left-edge . ,(- (framethickness) 1))
				 (right-edge . ,(- (framethickness) 1))
				 (top-edge . ,(- 0 (framethickness)))
				 (height . ,framethickness)
				 (class . title)
				 )
				;; Iconify gradient button
				(
				 (renderer . ,render-bg)
				 (render-scale . ,scale)
				 (foreground . ,fontcolor)
				 (font . ,glyph-font)
				 (text . ,iconify-glyph)
				 (x-justify . center)
				 (y-justify . center)
				 (left-edge . -1)
				 (top-edge . ,(- 0 (framethickness)))
				 (width . ,framethickness)
				 (height . ,framethickness)
				 (class . iconify-button)
				 )
				;; Close gradient button
				(
				 (renderer . ,render-bg)
				 (render-scale . ,scale)
				 (foreground . ,fontcolor)
				 (font . ,glyph-font)
				 (text . ,close-glyph)
				 (x-justify . center)
				 (y-justify . center)
				 (right-edge . -1)
				 (top-edge . ,(- 0 (framethickness)))
				 (width . ,framethickness)
				 (height . ,framethickness)
				 (class . close-button)
				 )
				)
			)
		 
     (transient-frame 
			`(
				;; Titlebar region
				(
				 (renderer . ,render-bg)
				 (render-scale . ,scale)
				 (foreground . ,fontcolor)
				 (text . ,(if thinmaker:transient-title window-name nil)) 
				 (x-justify . 10)
				 (y-justify . center)
				 (left-edge . -1)
				 (right-edge . -1)
				 (top-edge . ,(- 0 (transientthickness)))
				 (height . ,transientthickness)
				 (class . title)
				 )
				;; left outline
				(
				 (background . "black")
				 (left-edge . -1)
				 (width . 1)
				 (top-edge . 0)
				 (bottom-edge . 0)
				 )
				;; right outline
				(
				 (background . "black")
				 (right-edge . -1)
				 (width . 1)
				 (top-edge . 0)
				 (bottom-edge . 0)
				 )
				;; bottom bar
				(
				 (renderer . ,render-bg)
				 (render-scale . ,scale)
				 (left-edge . -1)
				 (right-edge . -1)
				 (bottom-edge . ,(- 0 (resizethickness)))
				 (height . ,resizethickness)
				 (class . bottom-border)
				 )
				)
			)
		 
     (shaped-transient-frame
			`(
				;; Titlebar region
				(
				 (renderer . ,render-bg)
				 (render-scale . ,scale)
				 (foreground . ,fontcolor)
				 (text . ,(if thinmaker:transient-title window-name nil)) 
				 (x-justify . 10)
				 (y-justify . center)
				 (left-edge . -1)
				 (right-edge . -1)
				 (top-edge . ,(- 0 (transientthickness)))
				 (height . ,transientthickness)
				 (class . title)
				 )
				)
			)
		 )
	
  (add-frame-style
	 'thinmaker
	 (lambda 
		 (w type)
		 (cond 
			(
			 (eq type 'shaped)
			 shaped-frame
			 )
			(
			 (eq type 'transient)
			 transient-frame
			 )
			(
			 (eq type 'shaped-transient)
			 shaped-transient-frame
			 )
			(
			 (eq type 'unframed)
			 nil-frame
			 )
			(t frame)
			)
		 )
	 )
	)
